@@ -35,10 +35,13 @@
 #include "glib-utils.h"
 #include "fr-archive.h"
 #include "fr-command.h"
+#include "fr-command-ace.h"
 #include "fr-command-ar.h"
 #include "fr-command-arj.h"
 #include "fr-command-cfile.h"
+#include "fr-command-cpio.h"
 #include "fr-command-iso.h"
+#include "fr-command-jar.h"
 #include "fr-command-lha.h"
 #include "fr-command-rar.h"
 #include "fr-command-rpm.h"
@@ -305,9 +308,15 @@ create_command_from_mime_type (FRArchive  *archive,
 		   is_mime_type (mime_type, "application/x-ar")) {
 		archive->command = fr_command_ar_new (archive->process, 
 						      filename);
+	} else if (is_mime_type (mime_type, "application/x-ace")) {
+		archive->command = fr_command_ace_new (archive->process, 
+						       filename);
 	} else if (is_mime_type (mime_type, "application/x-7zip")) {
 		archive->command = fr_command_7z_new (archive->process,
 						      filename);
+	} else if (is_mime_type (mime_type, "application/x-cpio")) {
+		archive->command = fr_command_cpio_new (archive->process,
+						        filename);						      
 	} else 
 		return FALSE;
 
@@ -368,6 +377,9 @@ get_mime_type_from_sniffer (const char *filename)
 	
 	if (file == NULL) 
                 return NULL;
+                
+        if (file_extension_is (filename, ".jar"))
+        	return NULL;
 	
 	n = fread (buffer, sizeof (char), sizeof (buffer) - 1, file);
 	buffer[n] = 0;
@@ -443,12 +455,17 @@ create_command_from_filename (FRArchive  *archive,
 
 	if (file_extension_is (filename, ".zip")
 	    || file_extension_is (filename, ".ear")
-	    || file_extension_is (filename, ".jar")
 	    || file_extension_is (filename, ".war")) {
 		archive->command = fr_command_zip_new (archive->process, 
 						       filename);
 		return (archive->command != NULL);
 	} 
+
+	if (file_extension_is (filename, ".jar")) {
+		archive->command = fr_command_jar_new (archive->process, 
+						       filename);
+		return (archive->command != NULL);
+	} 
 	
 	if (file_extension_is (filename, ".zoo")) {
 		archive->command = fr_command_zoo_new (archive->process,
@@ -512,12 +529,17 @@ create_command_from_filename (FRArchive  *archive,
 		if (file_extension_is (filename, ".lzo")) {
 			archive->command = fr_command_cfile_new (archive->process, filename, FR_COMPRESS_PROGRAM_LZOP);
 			archive->is_compressed_file = TRUE;
-			return (archive->command != NULL);
+			return (archive->command != NULL);			
 		} 
+		
+		if (file_extension_is (filename, ".cpio")) {
+			archive->command = fr_command_cpio_new (archive->process,
+							        filename);
+			return (archive->command != NULL);
+		}		
 	} 
 
 	if (loading) {
-		
 		if (file_extension_is (filename, ".bin")
 		    || file_extension_is (filename, ".sit")) {
 			archive->command = fr_command_unstuff_new (archive->process,
@@ -542,6 +564,12 @@ create_command_from_filename (FRArchive  *archive,
 							      filename);
 			return (archive->command != NULL);
 		} 
+
+		if (file_extension_is (filename, ".ace")) {
+			archive->command = fr_command_ace_new (archive->process, 
+							       filename);
+			return (archive->command != NULL);
+		} 		
 	}
 
 	return FALSE;
@@ -700,10 +728,11 @@ fr_archive_fake_load (FRArchive *archive)
 gboolean
 fr_archive_load (FRArchive   *archive, 
 		 const char  *filename,
+		 const char  *password,
 		 GError     **gerror)
 {
 	FRCommand  *tmp_command;
-	const char *mime_type;
+	const char *mime_type = NULL;
 
 	g_return_val_if_fail (archive != NULL, FALSE);
 
@@ -717,19 +746,20 @@ fr_archive_load (FRArchive   *archive,
 
 	archive->read_only = access (filename, W_OK) != 0;
 
-	if (archive->filename != NULL)
-		g_free (archive->filename);
-	archive->filename = g_strdup (filename);
+	if (archive->filename != filename) {
+		if (archive->filename != NULL)
+			g_free (archive->filename);
+		archive->filename = g_strdup (filename);
+	}
 
 	tmp_command = archive->command;
 
 	/* prefer mime-magic */
 
 	mime_type = get_mime_type_from_sniffer (filename);
-
 	if (mime_type == NULL)
 		mime_type = get_mime_type_from_content (filename);
-	
+
 	if ((mime_type == NULL)
 	    || ! create_command_from_mime_type (archive, filename, mime_type))
 		if (! create_command_from_filename (archive, filename, TRUE)) {
@@ -772,7 +802,7 @@ fr_archive_load (FRArchive   *archive,
 	archive->command->fake_load = fr_archive_fake_load (archive);
 
 	fr_process_clear (archive->process);
-	fr_command_list (archive->command);
+	fr_command_list (archive->command, password);
 	fr_process_start (archive->process);
 
 	return TRUE;
@@ -780,7 +810,8 @@ fr_archive_load (FRArchive   *archive,
 
 
 void
-fr_archive_reload (FRArchive *archive)
+fr_archive_reload (FRArchive  *archive,
+		   const char *password)
 {
 	g_return_if_fail (archive != NULL);
 	g_return_if_fail (archive->filename != NULL);
@@ -789,7 +820,7 @@ fr_archive_reload (FRArchive *archive)
 	archive->command->fake_load = fr_archive_fake_load (archive);
 
 	fr_process_clear (archive->process);
-	fr_command_list (archive->command);
+	fr_command_list (archive->command, password);
 	fr_process_start (archive->process);
 }
 
@@ -805,7 +836,7 @@ fr_archive_rename (FRArchive  *archive,
 		/* If the archive is a compressed file we have to reload it,
 		 * because in this case the 'content' of the archive changes 
 		 * too. */
-		fr_archive_load (archive, filename, NULL);
+		fr_archive_load (archive, filename, NULL, NULL);
 
 	else {
 		if (archive->filename != NULL)
@@ -1232,6 +1263,7 @@ fr_archive_add_with_wildcard (FRArchive     *archive,
 					     NO_BACKUP_FILES,
 					     NO_DOT_FILES,
 					     IGNORE_CASE,
+					     archive->command->propAddCanStoreFolders,
 					     add_with_wildcard__step2, 
 					     aww_data);
 }
@@ -1314,6 +1346,7 @@ fr_archive_add_directory (FRArchive     *archive,
 
 	return get_items_file_list_async (ad_data->dir_list,
 					  base_dir, 
+					  archive->command->propAddCanStoreFolders,
 					  add_directory__step2, 
 					  ad_data);
 }
@@ -1352,6 +1385,7 @@ fr_archive_add_items (FRArchive     *archive,
 
 	return get_items_file_list_async (ad_data->dir_list, 
 					  base_dir, 
+					  archive->command->propAddCanStoreFolders,
 					  add_directory__step2, 
 					  ad_data);
 }
@@ -1724,8 +1758,7 @@ fr_archive_extract (FRArchive  *archive,
 		GList *scan;
 
 		file_list = NULL;
-		scan = archive->command->file_list;
-		for (; scan; scan = scan->next) {
+		for (scan = archive->command->file_list; scan; scan = scan->next) {
 			FileData *fdata = scan->data;
 			file_list = g_list_prepend (file_list, g_strdup (fdata->original_path));
 		}
@@ -1745,6 +1778,15 @@ fr_archive_extract (FRArchive  *archive,
 		if (fdata == NULL)
 			continue;
 
+		if (((archive->command->file_type == FR_FILE_TYPE_TAR)
+		     || (archive->command->file_type == FR_FILE_TYPE_TAR_BZ)
+		     || (archive->command->file_type == FR_FILE_TYPE_TAR_BZ2)
+		     || (archive->command->file_type == FR_FILE_TYPE_TAR_GZ)
+		     || (archive->command->file_type == FR_FILE_TYPE_TAR_LZOP)
+		     || (archive->command->file_type == FR_FILE_TYPE_TAR_COMPRESS))
+		    && fdata->dir)
+			continue;
+
 		/* get the destination file path. */
 
 		if (! junk_paths)
@@ -1764,13 +1806,13 @@ fr_archive_extract (FRArchive  *archive,
 		
 		if (! archive->command->propExtractCanSkipOlder
 		    && skip_older 
-		    && path_is_file (dest_filename)
+		    && path_exists (dest_filename)
 		    && (fdata->modified < get_file_mtime (dest_filename)))
 			continue;
 
 		if (! archive->command->propExtractCanAvoidOverwrite
 		    && ! overwrite 
-		    && path_is_file (dest_filename))
+		    && path_exists (dest_filename))
 			continue;
 
 		filtered = g_list_prepend (filtered, fdata->original_path);
@@ -1778,7 +1820,7 @@ fr_archive_extract (FRArchive  *archive,
 
 	if (filtered == NULL) {
 		/* all files got filtered, do nothing. */
-		debug (DEBUG_INFO, "All files got filtered, do nothing.\n");
+		debug (DEBUG_INFO, "All files got filtered, nothing to do.\n");
 		if (extract_all) 
 			path_list_free (file_list);
 		return;
@@ -1862,11 +1904,13 @@ fr_archive_utils__get_file_name_ext (const char *filename)
 {
 	static char * ext[] = {
 		".7z",
+		".ace",
 		".ar",
 		".arj",
 		".bin",
 		".bz", 
-		".bz2", 
+		".bz2",
+		".cpio", 
 		".deb",
 		".ear",
 		".gz", 
